from __future__ import annotations

from prettyqt.qt import QtWidgets


class ItemEditorCreatorBase(QtWidgets.QItemEditorCreatorBase):
    """Abstract base class for implementing new item editor creators."""
